==> underscore.deferred-0.0.3.tgz: package/test/deferred.js <==
module("deferred" );

_.each( [ "", " - new operator" ], function( withNew ) {


  function createDeferred() {
    return withNew ? new _._Deferred() : _._Deferred();
  }

  test("_._Deferred" + withNew, function() {

    expect( 11 );

    var deferred,
      object,
      test;

    deferred = createDeferred();

    test = false;

    deferred.done( function( value ) {
      equals( value , "value" , "Test pre-resolve callback" );
      test = true;
    } );

    deferred.resolve( "value" );

    ok( test , "Test pre-resolve callbacks called right away" );

    test = false;

    deferred.done( function( value ) {
      equals( value , "value" , "Test post-resolve callback" );
      test = true;
    } );

    ok( test , "Test post-resolve callbacks called right away" );

    deferred.cancel();

    test = true;

    deferred.done( function() {
      ok( false , "Cancel was ignored" );
      test = false;
    } );

    ok( test , "Test cancel" );

    deferred = createDeferred().resolve();

    try {
      deferred.done( function() {
        throw "Error";
      } , function() {
        ok( true , "Test deferred do not cancel on exception" );
      } );
    } catch( e ) {
      strictEqual( e , "Error" , "Test deferred propagates exceptions");
      deferred.done();
    }

    test = "";
    deferred = createDeferred().done( function() {

      test += "A";

    }, function() {

      test += "B";

    } ).resolve();

    strictEqual( test , "AB" , "Test multiple done parameters" );

    test = "";

    deferred.done( function() {

      deferred.done( function() {

        test += "C";

      } );

      test += "A";

    }, function() {

      test += "B";
    } );

    strictEqual( test , "ABC" , "Test done callbacks order" );

    deferred = createDeferred();

    deferred.resolveWith( _ , [ document ] ).done( function( doc ) {
      ok( this === _ && arguments.length === 1 && doc === document , "Test fire context & args" );
    });

    // #8421
    deferred = createDeferred();
    deferred.resolveWith().done(function() {
      ok( true, "Test resolveWith can be called with no argument" );
    });
  });
} );

_.each( [ "", " - new operator" ], function( withNew ) {

  function createDeferred( fn ) {
    return withNew ? new _.Deferred( fn ) : _.Deferred( fn );
  }

  test("_.Deferred" + withNew, function() {

    expect( 8 );

    createDeferred().resolve().then( function() {
      ok( true , "Success on resolve" );
      ok( this.isResolved(), "Deferred is resolved" );
    }, function() {
      ok( false , "Error on resolve" );
    }).always( function() {
      ok( true , "Always callback on resolve" );
    });

    createDeferred().reject().then( function() {
      ok( false , "Success on reject" );
    }, function() {
      ok( true , "Error on reject" );
      ok( this.isRejected(), "Deferred is rejected" );
    }).always( function() {
      ok( true , "Always callback on reject" );
    });

    createDeferred( function( defer ) {
      ok( this === defer , "Defer passed as this & first argument" );
      this.resolve( "done" );
    }).then( function( value ) {
      strictEqual( value , "done" , "Passed function executed" );
    });
  });
} );

test( "_.Deferred.pipe - filtering (done)", function() {

  expect(4);

  var defer = _.Deferred(),
    piped = defer.pipe(function( a, b ) {
      return a * b;
    }),
    value1,
    value2,
    value3;

  piped.done(function( result ) {
    value3 = result;
  });

  defer.done(function( a, b ) {
    value1 = a;
    value2 = b;
  });

  defer.resolve( 2, 3 );

  strictEqual( value1, 2, "first resolve value ok" );
  strictEqual( value2, 3, "second resolve value ok" );
  strictEqual( value3, 6, "result of filter ok" );

  _.Deferred().reject().pipe(function() {
    ok( false, "pipe should not be called on reject" );
  });

  _.Deferred().resolve().pipe( _.noop ).done(function( value ) {
    strictEqual( value, undefined, "pipe done callback can return undefined/null" );
  });
});

test( "_.Deferred.pipe - filtering (fail)", function() {

  expect(4);

  var defer = _.Deferred(),
    piped = defer.pipe( null, function( a, b ) {
      return a * b;
    } ),
    value1,
    value2,
    value3;

  piped.fail(function( result ) {
    value3 = result;
  });

  defer.fail(function( a, b ) {
    value1 = a;
    value2 = b;
  });

  defer.reject( 2, 3 );

  strictEqual( value1, 2, "first reject value ok" );
  strictEqual( value2, 3, "second reject value ok" );
  strictEqual( value3, 6, "result of filter ok" );

  _.Deferred().resolve().pipe( null, function() {
    ok( false, "pipe should not be called on resolve" );
  } );

  _.Deferred().reject().pipe( null, _.noop ).fail(function( value ) {
    strictEqual( value, undefined, "pipe fail callback can return undefined/null" );
  });
});

test( "_.Deferred.pipe - deferred (done)", function() {

  expect(3);

  var defer = _.Deferred(),
    piped = defer.pipe(function( a, b ) {
      return _.Deferred(function( defer ) {
        defer.reject( a * b );
      });
    }),
    value1,
    value2,
    value3;

  piped.fail(function( result ) {
    value3 = result;
  });

  defer.done(function( a, b ) {
    value1 = a;
    value2 = b;
  });

  defer.resolve( 2, 3 );

  strictEqual( value1, 2, "first resolve value ok" );
  strictEqual( value2, 3, "second resolve value ok" );
  strictEqual( value3, 6, "result of filter ok" );
});

test( "_.Deferred.pipe - deferred (fail)", function() {

  expect(3);

  var defer = _.Deferred(),
    piped = defer.pipe( null, function( a, b ) {
      return _.Deferred(function( defer ) {
        defer.resolve( a * b );
      });
    } ),
    value1,
    value2,
    value3;

  piped.done(function( result ) {
    value3 = result;
  });

  defer.fail(function( a, b ) {
    value1 = a;
    value2 = b;
  });

  defer.reject( 2, 3 );

  strictEqual( value1, 2, "first reject value ok" );
  strictEqual( value2, 3, "second reject value ok" );
  strictEqual( value3, 6, "result of filter ok" );
});

test( "_.Deferred.pipe - context", function() {

  expect(4);

  var context = {};

  _.Deferred().resolveWith( context, [ 2 ] ).pipe(function( value ) {
    return value * 3;
  }).done(function( value ) {
    strictEqual( this, context, "custom context correctly propagated" );
    strictEqual( value, 6, "proper value received" );
  });

  var defer = _.Deferred(),
    piped = defer.pipe(function( value ) {
      return value * 3;
    });

  defer.resolve( 2 );

  piped.done(function( value ) {
    strictEqual( this.promise(), piped, "default context gets updated to latest defer in the chain" );
    strictEqual( value, 6, "proper value received" );
  });
});


test( "_.when" , function() {

  expect( 23 );

  // Some other objects
  _.each( {

    "an empty string": "",
    "a non-empty string": "some string",
    "zero": 0,
    "a number other than zero": 1,
    "true": true,
    "false": false,
    "null": null,
    "undefined": undefined,
    "a plain object": {}

  } , function( value, message ) {

    ok( _.isFunction( _.when( value ).done(function( resolveValue ) {
      strictEqual( resolveValue , value , "Test the promise was resolved with " + message );
    }).promise ) , "Test " + message + " triggers the creation of a new Promise" );

  } );

  ok( _.isFunction( _.when().done(function( resolveValue ) {
    strictEqual( resolveValue , undefined , "Test the promise was resolved with no parameter" );
  }).promise ) , "Test calling when with no parameter triggers the creation of a new Promise" );

  var cache, i;

  for( i = 1 ; i < 4 ; i++ ) {
    _.when( cache || _.Deferred( function() {
      this.resolve( i );
    }) ).done(function( value ) {
      strictEqual( value , 1 , "Function executed" + ( i > 1 ? " only once" : "" ) );
      cache = value;
    });
  }
});

test("_.when - joined", function() {

  expect(25);

  var deferreds = {
      value: 1,
      success: _.Deferred().resolve( 1 ),
      error: _.Deferred().reject( 0 ),
      futureSuccess: _.Deferred(),
      futureError: _.Deferred()
    },
    willSucceed = {
      value: true,
      success: true,
      error: false,
      futureSuccess: true,
      futureError: false
    };

  _.each( deferreds, function( defer1, id1 ) {
    _.each( deferreds, function( defer2, id2 ) {
      var shouldResolve = willSucceed[ id1 ] && willSucceed[ id2 ],
        expected = shouldResolve ? [ 1, 1 ] : [ 0, undefined ],
        code = id1 + "/" + id2;
      _.when( defer1, defer2 ).done(function( a, b ) {
        if ( shouldResolve ) {
          same( [ a, b ], expected, code + " => resolve" );
        }
      }).fail(function( a, b ) {
        if ( !shouldResolve ) {
          same( [ a, b ], expected, code + " => resolve" );
        }
      });
    } );
  } );
  deferreds.futureSuccess.resolve( 1 );
  deferreds.futureError.reject( 0 );
});

==> underscore.deferred-0.1.0.tgz: package/test/deferred.js <==
module("deferred" );

_.each( [ "", " - new operator" ], function( withNew ) {

  function createDeferred( fn ) {
    return withNew ? new _.Deferred( fn ) : _.Deferred( fn );
  }

  test("_.Deferred" + withNew, function() {

    expect( 22 );

    createDeferred().resolve().then( function() {
      ok( true , "Success on resolve" );
      ok( this.isResolved(), "Deferred is resolved" );
      strictEqual( this.state(), "resolved", "Deferred is resolved (state)" );
    }, function() {
      ok( false , "Error on resolve" );
    }).always( function() {
      ok( true , "Always callback on resolve" );
    });

    createDeferred().reject().then( function() {
      ok( false , "Success on reject" );
    }, function() {
      ok( true , "Error on reject" );
      ok( this.isRejected(), "Deferred is rejected" );
      strictEqual( this.state(), "rejected", "Deferred is rejected (state)" );
    }).always( function() {
      ok( true , "Always callback on reject" );
    });

    createDeferred( function( defer ) {
      ok( this === defer , "Defer passed as this & first argument" );
      this.resolve( "done" );
    }).then( function( value ) {
      strictEqual( value , "done" , "Passed function executed" );
    });

    _.each( "resolve reject".split( " " ), function( change ) {
      createDeferred( function( defer ) {
        strictEqual( defer.state(), "pending", "pending after creation" );
        var checked = 0;
        defer.progress(function( value ) {
          strictEqual( value, checked, "Progress: right value (" + value + ") received" );
        });
        for( checked = 0; checked < 3 ; checked++ ) {
          defer.notify( checked );
        }
        strictEqual( defer.state(), "pending", "pending after notification" );
        defer[ change ]();
        notStrictEqual( defer.state(), "pending", "not pending after " + change );
        defer.notify();
      });
    });
  });

} );


test( "_.Deferred - chainability", function() {

  var methods = "resolve reject notify resolveWith rejectWith notifyWith done fail progress then always".split( " " ),
    defer = _.Deferred();

  expect( methods.length );

  _.each( methods, function( method ) {
    var object = { m: defer[ method ] };
    strictEqual( object.m(), object, method + " is chainable" );
  });
});


test( "_.Deferred.pipe - filtering (done)", function() {

  expect(4);

  var defer = _.Deferred(),
    piped = defer.pipe(function( a, b ) {
      return a * b;
    }),
    value1,
    value2,
    value3;

  piped.done(function( result ) {
    value3 = result;
  });

  defer.done(function( a, b ) {
    value1 = a;
    value2 = b;
  });

  defer.resolve( 2, 3 );

  strictEqual( value1, 2, "first resolve value ok" );
  strictEqual( value2, 3, "second resolve value ok" );
  strictEqual( value3, 6, "result of filter ok" );

  _.Deferred().reject().pipe(function() {
    ok( false, "pipe should not be called on reject" );
  });

  _.Deferred().resolve().pipe( _.noop ).done(function( value ) {
    strictEqual( value, undefined, "pipe done callback can return undefined/null" );
  });
});

test( "_.Deferred.pipe - filtering (fail)", function() {

  expect(4);

  var defer = _.Deferred(),
    piped = defer.pipe( null, function( a, b ) {
      return a * b;
    } ),
    value1,
    value2,
    value3;

  piped.fail(function( result ) {
    value3 = result;
  });

  defer.fail(function( a, b ) {
    value1 = a;
    value2 = b;
  });

  defer.reject( 2, 3 );

  strictEqual( value1, 2, "first reject value ok" );
  strictEqual( value2, 3, "second reject value ok" );
  strictEqual( value3, 6, "result of filter ok" );

  _.Deferred().resolve().pipe( null, function() {
    ok( false, "pipe should not be called on resolve" );
  } );

  _.Deferred().reject().pipe( null, _.noop ).fail(function( value ) {
    strictEqual( value, undefined, "pipe fail callback can return undefined/null" );
  });
});

test( "_.Deferred.pipe - filtering (progress)", function() {

  expect(3);

  var defer = _.Deferred(),
    piped = defer.pipe( null, null, function( a, b ) {
      return a * b;
    } ),
    value1,
    value2,
    value3;

  piped.progress(function( result ) {
    value3 = result;
  });

  defer.progress(function( a, b ) {
    value1 = a;
    value2 = b;
  });

  defer.notify( 2, 3 );

  strictEqual( value1, 2, "first progress value ok" );
  strictEqual( value2, 3, "second progress value ok" );
  strictEqual( value3, 6, "result of filter ok" );
});

test( "_.Deferred.pipe - deferred (done)", function() {

  expect(3);

  var defer = _.Deferred(),
    piped = defer.pipe(function( a, b ) {
      return _.Deferred(function( defer ) {
        defer.reject( a * b );
      });
    }),
    value1,
    value2,
    value3;

  piped.fail(function( result ) {
    value3 = result;
  });

  defer.done(function( a, b ) {
    value1 = a;
    value2 = b;
  });

  defer.resolve( 2, 3 );

  strictEqual( value1, 2, "first resolve value ok" );
  strictEqual( value2, 3, "second resolve value ok" );
  strictEqual( value3, 6, "result of filter ok" );
});

test( "_.Deferred.pipe - deferred (fail)", function() {

  expect(3);

  var defer = _.Deferred(),
    piped = defer.pipe( null, function( a, b ) {
      return _.Deferred(function( defer ) {
        defer.resolve( a * b );
      });
    } ),
    value1,
    value2,
    value3;

  piped.done(function( result ) {
    value3 = result;
  });

  defer.fail(function( a, b ) {
    value1 = a;
    value2 = b;
  });

  defer.reject( 2, 3 );

  strictEqual( value1, 2, "first reject value ok" );
  strictEqual( value2, 3, "second reject value ok" );
  strictEqual( value3, 6, "result of filter ok" );
});

test( "_.Deferred.pipe - deferred (progress)", function() {

  expect(3);

  var defer = _.Deferred(),
    piped = defer.pipe( null, null, function( a, b ) {
      return _.Deferred(function( defer ) {
        defer.resolve( a * b );
      });
    } ),
    value1,
    value2,
    value3;

  piped.done(function( result ) {
    value3 = result;
  });

  defer.progress(function( a, b ) {
    value1 = a;
    value2 = b;
  });

  defer.notify( 2, 3 );

  strictEqual( value1, 2, "first progress value ok" );
  strictEqual( value2, 3, "second progress value ok" );
  strictEqual( value3, 6, "result of filter ok" );
});

test( "_.Deferred.pipe - context", function() {

  expect(4);

  var context = {};

  _.Deferred().resolveWith( context, [ 2 ] ).pipe(function( value ) {
    return value * 3;
  }).done(function( value ) {
    strictEqual( this, context, "custom context correctly propagated" );
    strictEqual( value, 6, "proper value received" );
  });

  var defer = _.Deferred(),
    piped = defer.pipe(function( value ) {
      return value * 3;
    });

  defer.resolve( 2 );

  piped.done(function( value ) {
    strictEqual( this.promise(), piped, "default context gets updated to latest defer in the chain" );
    strictEqual( value, 6, "proper value received" );
  });
});

test( "_.when" , function() {

  expect( 23 );

  // Some other objects
  _.each( {

    "an empty string": "",
    "a non-empty string": "some string",
    "zero": 0,
    "a number other than zero": 1,
    "true": true,
    "false": false,
    "null": null,
    "undefined": undefined,
    "a plain object": {}

  } , function(  value, message ) {

    ok( _.isFunction( _.when( value ).done(function( resolveValue ) {
      strictEqual( resolveValue , value , "Test the promise was resolved with " + message );
    }).promise ) , "Test " + message + " triggers the creation of a new Promise" );

  } );

  ok( _.isFunction( _.when().done(function( resolveValue ) {
    strictEqual( resolveValue , undefined , "Test the promise was resolved with no parameter" );
  }).promise ) , "Test calling when with no parameter triggers the creation of a new Promise" );

  var cache, i;

  for( i = 1 ; i < 4 ; i++ ) {
    _.when( cache || _.Deferred( function() {
      this.resolve( i );
    }) ).done(function( value ) {
      strictEqual( value , 1 , "Function executed" + ( i > 1 ? " only once" : "" ) );
      cache = value;
    });
  }
});

test("_.when - joined", function() {

  expect(53);

  var deferreds = {
      value: 1,
      success: _.Deferred().resolve( 1 ),
      error: _.Deferred().reject( 0 ),
      futureSuccess: _.Deferred().notify( true ),
      futureError: _.Deferred().notify( true ),
      notify: _.Deferred().notify( true )
    },
    willSucceed = {
      value: true,
      success: true,
      futureSuccess: true
    },
    willError = {
      error: true,
      futureError: true
    },
    willNotify = {
      futureSuccess: true,
      futureError: true,
      notify: true
    };

  _.each( deferreds, function( defer1, id1 ) {
    _.each( deferreds, function( defer2, id2 ) {
      var shouldResolve = willSucceed[ id1 ] && willSucceed[ id2 ],
        shouldError = willError[ id1 ] || willError[ id2 ],
        shouldNotify = willNotify[ id1 ] || willNotify[ id2 ],
        expected = shouldResolve ? [ 1, 1 ] : [ 0, undefined ],
          expectedNotify = shouldNotify && [ willNotify[ id1 ], willNotify[ id2 ] ],
          code = id1 + "/" + id2;

      var promise = _.when( defer1, defer2 ).done(function( a, b ) {
        if ( shouldResolve ) {
          deepEqual( [ a, b ], expected, code + " => resolve" );
        } else {
          ok( false ,  code + " => resolve" );
        }
      }).fail(function( a, b ) {
        if ( shouldError ) {
          deepEqual( [ a, b ], expected, code + " => reject" );
        } else {
          ok( false ,  code + " => reject" );
        }
      }).progress(function progress( a, b ) {
        deepEqual( [ a, b ], expectedNotify, code + " => progress" );
      });
    } );
  } );
  deferreds.futureSuccess.resolve( 1 );
  deferreds.futureError.reject( 0 );
});
